import React, {Component} from "react"
import {Redirect, Link} from "react-router-dom"
import axios from "axios"

import LinkInClass from "../components/LinkInClass"
import {ACCESS_LEVEL_GUEST, SERVER_HOST} from "../config/global_constants"


export default class Login extends Component
{
    constructor(props)
    {
        super(props)
        
        this.state = {
            email:"",
            password:"",
            wasSubmittedAtLeastOnce:false,
            isLoggedIn:false
        }
    }
    
    
    handleChange = (e) => 
    {
        this.setState({[e.target.name]: e.target.value})
    }
    
    
    handleSubmit = (e) => 
    {
        this.setState({ wasSubmittedAtLeastOnce: true })
        const formInputsState = this.validate()
    
        if (Object.keys(formInputsState).every(index => formInputsState[index])) 
        {
            axios.defaults.withCredentials = true // needed for sessions to work
            axios.post(`${SERVER_HOST}/users/login/${this.state.email}/${this.state.password}`)
            .then(res => 
            {     
                // default if not logged in
                sessionStorage.name = "GUEST"
                sessionStorage.accessLevel = ACCESS_LEVEL_GUEST // guest has accessLevel of 0
                if(res.data)
                {
                    if (res.data.errorMessage)
                    {
                        console.log(res.data.errorMessage)    
                    }
                    else // user successfully logged in
                    { 
                        console.log("User logged in")
                    
                        sessionStorage.name = res.data.name
                        sessionStorage.accessLevel = res.data.accessLevel
                    
                        this.setState({isLoggedIn:true})
                    }        
                }
                else
                {
                    console.log("Login failed")
                }
            })        
        }
        else // invalid inputs in form
        {
            e.preventDefault()
            return    
        }        
    }


    validateEmail()
    {    
        // valid email pattern
        const pattern = /^(([^<>()\[\]\\.,:\s@"]+(\.[^<>()\[\]\\.,:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/  //eslint-disable-line
        return pattern.test(String(this.state.email).toLowerCase())
    }


    validatePassword()
    {    
        const pattern = /^(?=.*[0-9])(?=.*[a-z])(?=.*[A-Z])(?=.*[£!#€$%^&*]).{10,}$/  //eslint-disable-line
        return pattern.test(String(this.state.password)) 
    }


    validate() 
    {
        return {
            email: this.validateEmail(),
            password: this.validatePassword()
        }
    }


    render()
    {
        let errorMessage = ""
        if(this.state.wasSubmittedAtLeastOnce)
        {
            errorMessage = <div className="error">Login Details are incorrect<br/></div>
        }
                 
        return (
            <form className="form-container" noValidate = {true} id = "loginOrRegistrationForm">
                <h2>Login</h2>
                
                {this.state.isLoggedIn ? <Redirect to="/DisplayAllCars"/> : null}                         
                      
                {errorMessage}
                
                <input 
                    type = "email" 
                    name = "email" 
                    placeholder = "Email"
                    autoComplete="email"
                    value={this.state.email} 
                    onChange={this.handleChange}
                />
                    
                <input 
                    type = "password" 
                    name = "password" 
                    placeholder = "Password"
                    autoComplete="password"
                    value={this.state.password} 
                    onChange={this.handleChange}
                /><br/>
                
                <LinkInClass value="Login" className="green-button" onClick={this.handleSubmit}/> 
                <Link className="red-button" to={"/DisplayAllCars"}>Cancel</Link>                                      
            </form>
        )
    }
}